/*
 * Tagger.java
 *
 * Copyright (C) 1998-2002 Peter Graves
 * $Id: Tagger.java,v 1.3 2002/12/08 01:25:59 piso Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.armedbear.j;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.List;

public abstract class Tagger implements Constants, Runnable
{
    public static final char separatorChar = 0;

    protected SystemBuffer buffer;

    protected Tagger(SystemBuffer buffer)
    {
        this.buffer = buffer;
    }

    public void writeTags(Writer writer)
    {
        if (buffer == null)
            return;
        List tags = buffer.getTags();
        if (tags == null)
            return;
        File file = buffer.getFile();
        if (file == null)
            return;
        final String canonicalPath = file.canonicalPath();
        try {
            final int limit = tags.size();
            for (int i = 0; i < limit; i++) {
                LocalTag localTag = (LocalTag) tags.get(i);
                if (localTag != null) {
                    switch (localTag.getType()) {
                        case TAG_INTERFACE:
                        case TAG_CLASS:
                        case TAG_METHOD:
                        case TAG_EXPLICIT:
                            writer.write(localTag.getName());
                            writer.write(separatorChar);
                            writer.write(canonicalPath);
                            writer.write(separatorChar);
                            writer.write(localTag.getLine().getText());
                            final String canonicalSignature =
                                localTag.getCanonicalSignature();
                            if (canonicalSignature != null) {
                                writer.write(separatorChar);
                                writer.write(canonicalSignature);
                            }
                            writer.write('\n');
                            break;
                        default:
                            break;
                    }
                }
            }
            writer.flush();
        }
        catch (IOException e) {
            Log.error(e);
        }
    }

    public abstract void run();
}
